package ic.dfa;

import ic.cfg.ControlFlowGraph;
import ic.tac.TACPropagatingVisitor;
import ic.tac.TBinary;
import ic.tac.TCopy;
import ic.tac.TInstr;


/**
 * Same as ReachableAnalysis, but illustrates how to leverage the TAC
 * visitor pattern (if you have one) to implement the transfer functions
 * in a relatively elegant way.
 */
public class ReachableAnalysisWithVisitor extends DataFlowAnalysis<Boolean> implements TACPropagatingVisitor<Boolean,Boolean> {

	public ReachableAnalysisWithVisitor(ControlFlowGraph cfg) {
		super(cfg);
	}

	public Boolean boundary() {
		return true;
	}

	public Boolean top() {
		return false;
	}

	public boolean equals(Boolean t1, Boolean t2) {
		return t1.equals(t2);
	}

	public boolean isForward() {
		return true;
	}

	public Boolean meet(Boolean t1, Boolean t2) {
		return t1 || t2;
	}

	public Boolean transfer(TInstr instr, Boolean t) {
		return instr.accept(this, t);
	}

	// Just write each transfer function for an instruction 
	// form as a visit method: 
	
	public Boolean visit(TBinary x, Boolean t) {
		return t;
	}

	public Boolean visit(TCopy x, Boolean t) {
		return t;
	}

	// ... Rest of visit methods here ...

}
