package ic.opt;

import ic.ast.*;

/**
 * To create an optimization, simply subclass this class and define
 * optimize(md) to compute any dataflow information necessary to
 * perform the optimization on md and then modify the method's TAC
 * list. You can either provide methods in your TACList class to
 * modify an existing list, or you can construct a new list to replace
 * the old one.
 */
public abstract class Optimization {

	/**
	 * Apply the optimization to each Method in p.
	 */
	public void optimize(Program p) {

		/* 
		 * Insert your own code to call optimize on each MethodDecl.
		 * You can use a visitor, or just iterate over the decls in each
		 * ClassDecl and look for MethodDecls.
		 */

		for (ClassDecl c : p.getClasses()) {
			for (TypeDeclElem elem : c.getElems()) {
				if (elem instanceof MethodDecl) {
				    optimize((MethodDecl) elem);
				}
			}
		}
	}


	/**
	 * Apply the optimization to the method md.
	 */
	public abstract void optimize(MethodDecl md);
}
